unit dir_fns;
{Copyright (C) 1994, Jon Shemitz <jon@armory.com> }

interface

uses BigString;

type
  FileName = array[1..12] of char;
  {Not a Pascal string and not a C string!  No terminator for full eleven-char
   names!  Blank padding for short names!}
  FileNameHeader = object
                   N: word;
                   end;
  FileNames = object (FileNameHeader)
              Name: array[1..$FFF0 div SizeOf(FileName)] of FileName;
              end;
  FileNamesPtr = ^ FileNames;
  FileNamesPtrPtr = ^ FileNamesPtr;

function New_FileNames(N: word): FileNamesPtr;

procedure Dispose_FileNames(Ptr: FileNamesPtr);

function ScanDir( Names:    FileNamesPtrPtr;
                  Clusters: BigStringPtrPtr;
                  Path, Mask: string; Attr: byte;
                  ClusterSize, DiskClusters: word): LongInt;
                  {Returns total clusters}

implementation

uses DOS;

function New_FileNames(N: word): FileNamesPtr;
var
  Size:   word;
  Result: FileNamesPtr;
begin
  Size := SizeOf(FileNameHeader) + SizeOf(FileName) * N;
  GetMem(Result, Size); {Will just crash if no mem!}
  FillChar(Result^, Size, ' ');
  Result^.N := N;
  New_FileNames := Result;
end;

procedure Dispose_FileNames(Ptr: FileNamesPtr);
begin
  FreeMem(Ptr, SizeOf(FileNameHeader) + SizeOf(FileName) * Ptr^.N);
end;

function ScanDir( Names:    FileNamesPtrPtr;
                  Clusters: BigStringPtrPtr;
                  Path, Mask: string; Attr: byte;
                  ClusterSize, DiskClusters: word): LongInt; {Total clusters}
var
  N, Size:       word;

  TotalClusters: LongInt;

  NameP:         FileNamesPtr;
  SizeP:         BigStringPtr;
  Search:        DOS.SearchRec;
  FullMask:      string[128];
begin
  {First, create the FullMask}
  if Path = '' then RunError(201); {Must specify a path}
  if Path[Length(Path)] in ['\', '/']
    then FullMask := Path + Mask
    else FullMask := Path + '\' + Mask;
  {I recently found that most DOS fns have always accepted the Unix style
   slash.  You can even mix \ and / ....}

  {Then, find how many files there are}
  N := 0;
  TotalClusters := 0;
  FindFirst(FullMask, Attr, Search);
  while DosError = 0 do
    begin
    if Search.Size <> 0 then
      begin
      Size := (Search.Size + ClusterSize - 1) div ClusterSize;
      if Size > DiskClusters then RunError(201); {File too large}
      Inc(TotalClusters, Size);
      Inc(N);
      end;
    FindNext(Search);
    end;

  {Then, allocate results}
  NameP := Nil;
  SizeP := Nil;
  if Names <> Nil then    {save sizes}
    begin
    Names^    := New_FileNames(N);
    NameP     := Names^;
    end;
  if Clusters <> Nil then {save names}
    begin
    Clusters^ := New_BigString(_Wrd, N);
    SizeP     := Clusters^;
    end;

  {Finally, scan again and fill in results}
  if (Names <> Nil) or (Clusters <> Nil) then
    begin
    N             := 1;
    FindFirst(FullMask, Attr, Search);
    while DosError = 0 do
      begin
      if Search.Size <> 0 then
        begin
        if SizeP <> Nil then
          SizeP^.Data.Wrd[N] := (Search.Size + ClusterSize - 1) div ClusterSize;
        if NameP <> Nil then
          Move(Search.Name[1], NameP^.Name[N], Length(Search.Name));
        Inc(N);
        end;
      FindNext(Search);
      end;
    end;

  ScanDir := TotalClusters;
end;

end.